@@ -18,8 +18,8 @@
 //! - Sends 100-Continue
 //! - Limits number of threads and connections
 //! - Modular: roll your own logging, write custom versions of internal methods, etc.
-//! - No macros or complicated type params
-//! - Good test coverage (63%) - TODO: Update.
+//! - N macros or complicated type params
+//! - Good test coverage (63%)
 //!
 //! # Limitations
 //! - New, not proven in production.
@@ -75,11 +75,7 @@
 //!
 //! let state = Arc::new(State {});
 //! let request_handler = move |req: Request| {
-//!     print_log_response(
-//!         req.method().to_string(),
-//!         req.url().clone(),
-//!         handle_req(state, &req),
-//!     )
+//!     print_log_response(&req, handle_req(state, &req))
 //! };
 //! let cache_dir = TempDir::new().unwrap();
 //! safina_timer::start_timer_thread();
@@ -103,43 +99,72 @@
 //! # Cargo Geiger Safety Report
 //! # Alternatives
 //!
-//! |                     |    |    |     |    |    |    |    |    |     |    |
-//! |---------------------|----|----|-----|----|----|----|----|----|-----|----|
-//! |  | beatrice | [rouille](https://crates.io/crates/rouille) | [trillium](https://crates.io/crates/trillium) | [tide](https://crates.io/crates/tide) | [axum](https://crates.io/crates/axum) | [poem](https://crates.io/crates/poem) | [warp](https://crates.io/crates/warp) | [thruster](https://crates.io/crates/thruster) | [rocket](https://crates.io/crates/rocket) | [gotham](https://crates.io/crates/gotham) |
-//! | Well-tested         | ❓ | ❌ | ❌ | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ |
-//! | Blocking handlers   | 🟢 | 🟢 | ❌ | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ |
-//! | Async handlers      | ❌ | ❌ | 🟢 | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ |
-//! | 100-continue        | 🟢 | 🟢 | 🟢 | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ |
-//! | Thread limit        | 🟢 | [❌](https://github.com/tiny-http/tiny-http/issues/221) | 🟢 | ❓ | ❓ | 🟢 | ❓ | ❓ | ❓ | ❓ |
-//! | Connection limit    | 🟢 | ❌ | ❌ | ❓ | ❓ | ❌ | ❓ | ❓ | ❓ | ❓ |
-//! | Caches payloads     | 🟢 | ❌ | ❌ | ❓ | ❓ | [❌](https://github.com/poem-web/poem/issues/75) | ❓ | ❓ | ❓ | ❓ |
-//! | Request timeouts    | ❌ | ❌ | ❌ | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ |
-//! | Custom logging      | 🟢 | 🟢 | 🟢 | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ |
-//! | Contains no unsafe  | 🟢 | 🟢 | ❌ | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ |
-//! | No unsafe deps      | ❌ | ❌ | ❌ | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ |
-//! | age (years)         | 0  | 6  | 1   | 3  | 0  | 1  | ❓ | ❓ | ❓ | 5 |
-//! | TLS                 | ❌ | ❌ | 🟢 | ❓ | ❓ | 🟢 | ❓ | ❓ | ❓ | ❓ |
-//! | ACME certs          | ❌ | ❌ | ❌ | ❓ | ❓ | 🟢 | ❓ | ❓ | ❓ | ❓ |
-//! | SSE                 | ❌ | ❌ | ❓ | ❓ | ❓ | 🟢 | ❓ | ❓ | ❓ | ❓ |
-//! | Websockets          | ❌ | 🟢 | ❓ | ❓ | ❓ | 🟢 | ❓ | ❓ | ❓ | ❓ |
-//! | Streaming response: |    |    |     |    |    |    |    |    |     | ❓ |
-//! | - impl `AsyncRead`  | ❌ | ❌ | ❓ | ❓ | ❓ | 🟢 | ❓ | ❓ | ❓ | ❓ |
-//! | - `AsyncWrite`      | ❌ | ❌ | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ |
-//! | - impl `Read`       | ❌ | 🟢 | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ |
-//! | - channel           | ❌ | ❌ | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ | ❓ |
-//! | Custom routing      | 🟢 | 🟢 | ❓ | ❓ | ❓ | ❌ | ❓ | ❓ | ❓ | ❓ |
-//! | Usable sans macros  | 🟢 | 🟢 | ❓ | ❓ | ❓ | ❌ | ❓ | ❓ | ❓ | ❓ |
-//! | Shutdown for tests  | ❓ | ❓ | ❓ | ❓ | ❓ | 🟢 | ❓ | ❓ | ❓ | ❓ |
-//! | Graceful shutdown   | ❓ | ❓ | ❓ | ❓ | ❓ | 🟢 | ❓ | ❓ | ❓ | ❓ |
-//! | Rust stable         | ❓ | ❓ | ❓ | ❓ | ❓ | 🟢 | ❓ | ❓ | ❌ | ❓ |
+//! |  | beatrice | [rouille](https://crates.io/crates/rouille) | [trillium](https://crates.io/crates/trillium) | [tide](https://crates.io/crates/tide) | [axum](https://crates.io/crates/axum) | [poem](https://crates.io/crates/poem) |
+//! |---------------------|----|----|----|----|----|----|
+//! | Well-tested         | [NO](https://github.com/mleonhard/beatrice-rs/issues/1) | NO | [NO](https://github.com/trillium-rs/trillium/discussions/181) | NO | NO | NO |
+//! | Blocking handlers   | ✓  | ✓  | NO | NO | NO | ✓  |
+//! | Async handlers      | NO | NO | ✓  | ✓  | ✓  | ✓  |
+//! | 100-continue        | ✓  | ✓  | ✓  | [NO](https://github.com/http-rs/tide/issues/878) | ✓ | ✓ |
+//! | Thread limit        | ✓  | [NO](https://github.com/tiny-http/tiny-http/issues/221) | ✓ | ✓ | ✓ | ✓ |
+//! | Connection limit    | ✓  | NO | ✓  | NO | NO | NO |
+//! | Caches payloads     | ✓  | NO | NO | NO | NO | [NO](https://github.com/poem-web/poem/issues/75) |
+//! | Request timeouts    | NO | NO | NO | NO | NO | NO |
+//! | Custom logging      | ✓  | ✓  | ✓  | NO | ✓  | ✓  |
+//! | Unsafe-free         | ✓  | ✓  | NO | NO | NO | ✓  |
+//! | Unsafe-free deps    | NO | NO | NO | NO | NO | NO |
+//! | age (years)         | 0  | 6  | 1  | 3  | 0  | 1  |
+//! | TLS                 | NO | NO | ✓  | ✓  | ✓  | ✓  |
+//! | ACME certs          | NO | NO | NO | NO | NO | [o](https://docs.rs/poem/1.3.29/poem/listener/acme/index.html) |
+//! | SSE                 | ✓  | NO | [NO](https://github.com/trillium-rs/trillium/issues/39) | ✓ | ✓ | ✓ |
+//! | Websockets          | NO | ✓  | ✓  | ✓  | ✓  | ✓  |
+//! | Streaming response: |    |    |    |    |    |    |
+//! | - impl `AsyncRead`  | NO | NO | ✓  | ✓  | ✓  | ✓  |
+//! | - `AsyncWrite`      | NO | NO | NO | NO | NO | NO |
+//! | - impl `Read`       | NO | ✓  | NO | NO | NO | NO |
+//! | - channel           | NO | NO | NO | NO | ✓  | NO |
+//! | Custom routing      | ✓  | ✓  | ✓  | NO | ✓  | ✓  |
+//! | Usable sans macros  | ✓  | ✓  | ✓  | ✓  | ✓  | NO |
+//! | Shutdown for tests  | ✓  | ✓  | ✓  | [NO](https://github.com/http-rs/tide/issues/876) | ✓ | ✓ |
+//! | Graceful shutdown   | NO | ✓  | ✓  | [NO](https://github.com/http-rs/tide/issues/528) | ✓ | ✓ |
+//! | Rust stable         | ✓  | ✓  | ✓  | ✓  | ✓  | ✓  |
+//!
+//! |  | beatrice | [warp](https://crates.io/crates/warp) | [thruster](https://crates.io/crates/thruster) | [rocket](https://crates.io/crates/rocket) | [gotham](https://crates.io/crates/gotham) |
+//! |---------------------|----|----|----|----|----|
+//! | Well-tested         | [NO](https://github.com/mleonhard/beatrice-rs/issues/1) | ? | ? | ? | ? |
+//! | Blocking handlers   | ✓  | ?  | ?  | ?  | ?  |
+//! | Async handlers      | NO | ?  | ?  | ?  | ?  |
+//! | 100-continue        | ✓  | ?  | ?  | ?  | ?  |
+//! | Thread limit        | ✓  | ?  | ?  | ?  | ?  |
+//! | Connection limit    | ✓  | ?  | ?  | ?  | ?  |
+//! | Caches payloads     | ✓  | ?  | ?  | ?  | ?  |
+//! | Request timeouts    | NO | ?  | ?  | ?  | ?  |
+//! | Custom logging      | ✓  | ?  | ?  | ?  | ?  |
+//! | Unsafe-free         | ✓  | ?  | ?  | ?  | ?  |
+//! | Unsafe-free deps    | NO | ?  | ?  | ?  | ?  |
+//! | age (years)         | 0  | ?  | ?  | ?  | 5  |
+//! | TLS                 | NO | ?  | ?  | ?  | ?  |
+//! | ACME certs          | NO | ?  | ?  | ?  | ?  |
+//! | SSE                 | ✓  | ?  | ?  | ?  | ?  |
+//! | Websockets          | NO | ?  | ?  | ?  | ?  |
+//! | Streaming response: |    |    |    |    |    |
+//! | - impl `AsyncRead`  | NO | ?  | ?  | ?  | ?  |
+//! | - `AsyncWrite`      | NO | ?  | ?  | ?  | ?  |
+//! | - impl `Read`       | NO | ?  | ?  | ?  | ?  |
+//! | - channel           | NO | ?  | ?  | ?  | ?  |
+//! | Custom routing      | ✓  | ?  | ?  | ?  | ?  |
+//! | Usable sans macros  | ✓  | ?  | ?  | ?  | ?  |
+//! | Shutdown for tests  | ✓  | ?  | ?  | ?  | ?  |
+//! | Graceful shutdown   | NO | ?  | ?  | ?  | ?  |
+//! | Rust stable         | ✓  | ?  | ?  | NO | ?  |
 //!
 //! # Changelog
+//! - v0.2.0 - Make `print_log_response` easier to use.
 //! - v0.1.0 - First published version
 //!
 //! # TO DO
 //! - Fix limitations above
 //! - Support [HEAD](https://developer.mozilla.org/en-US/docs/Web/HTTP/Methods/HEAD)
-//!   responses that have Content-Length set and no body.
+//!   responses that have Content-Length set and N body.
 //! - Update alternatives table
 //! - Add other servers from <https://www.arewewebyet.org/topics/frameworks/> to alternatives table
 //!
@@ -221,15 +246,10 @@ use async_net::TcpListener;
 use permit::Permit;
 use std::net::SocketAddr;
 use std::path::PathBuf;
-use url::Url;
 
 #[allow(clippy::needless_pass_by_value)]
 #[must_use]
-pub fn print_log_response(
-    method: String,
-    url: Url,
-    result: Result<Response, Response>,
-) -> Response {
+pub fn print_log_response(req: &Request, result: Result<Response, Response>) -> Response {
     let response = result.unwrap_or_else(|e| e);
     if !response.is_get_body_and_reprocess() {
         println!(
@@ -239,8 +259,8 @@ pub fn print_log_response(
             } else {
                 "INFO"
             },
-            method,
-            url.path(),
+            req.method(),
+            req.url().path(),
             response.code,
             if let Some(len) = response.body.len() {
                 format!("len={}", len)
@@ -266,7 +286,7 @@ impl HttpServerBuilder {
     /// - Picks a random port
     /// - 100 max connections
     /// - 64 KiB small body length
-    /// - no cache dir, server rejects large request bodies
+    /// - N cache dir, server rejects large request bodies
     #[allow(clippy::new_without_default)]
     #[must_use]
     pub fn new() -> Self {
